/*
    Copyright 2004,2005,2006 Luigi Auriemma

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    http://www.gnu.org/licenses/gpl.txt

The 4 bytes starting at offset 0x1444 of the QDF file contain the
encoded password used to "protect" the file.
The next 8 bytes instead contain the password for the operations.

The QDF file in fact is not encrypted but as only some bytes that
contain the password, so setting them to 0 will give full access
to the file.

*/


#include <stdio.h>
#include <stdlib.h>



#define VER     "0.1"
#define QSIGN   0x8fbd9eac
#define Q98SIGN 0x00010001
#define PWDOFF  0x1444  /* offset of password's bytes */
#define NUM     12      /* number of bytes to set to zero */



void std_err(void);



int main(int argc, char *argv[]) {
    FILE            *fd;
    int             i;
    unsigned int    sign;


    setbuf(stdout, NULL);

    fputs("\n"
        "Quicken 6/'98 data file: reset of all passwords "VER"\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n"
        "\n", stdout);

    if(argc < 2) {
        printf("\nUsage: %s <file.QDF>\n"
            "\n"
            "Note: I suggest you to do a backup copy of your QDF file before removing the password!\n"
            "\n", argv[0]);
        exit(1);
    }

    fd = fopen(argv[1], "r+b");
    if(!fd) std_err();

    fread((void *)&sign, 4, 1, fd);
    if(sign != QSIGN) {
        fputs("\nThis file is not a valid Quicken file\n", stdout);
        exit(1);
    }
    fread((void *)&sign, 4, 1, fd);
    if(sign != Q98SIGN) {
        fputs("\nThis file is not a Quicken 6/'98 file (the only format supported here)\n", stdout);
        exit(1);
    }

    if(fseek(fd, PWDOFF, SEEK_SET) < 0) std_err();

    for(i = 0; i < NUM; i++) {
        if(fgetc(fd)) break;
    }
    if(i == NUM) {
        fputs("\nFile is already password free, no changes made\n", stdout);
        fclose(fd);
        return(0);
    }
    i++;

        /* reset */
    fseek(fd, -i, SEEK_CUR);
    for(i = NUM; i; i--) fputc(0x00, fd);

    fclose(fd);
    printf("\nPasswords removed\n");
    return(0);
}



void std_err(void) {
    perror("\nError");
    exit(1);
}


